<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\NcrRecordAttachment;

class NcrRecordAttachmentController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('ncr_record'), $filename);

            $ncr_record_attachment = new NcrRecordAttachment();
            $ncr_record_attachment->ncr_id = 0;
            $ncr_record_attachment->path = $filename;
            $ncr_record_attachment->token = $token;
            $ncr_record_attachment->save();

            $responseBody = $this->responseBody(true, "NcrRecordAttachmentController", "uploaded", $ncr_record_attachment->token);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "NcrRecordAttachmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
