<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\WeeklyLqr;
use Modules\Qc\Entities\WeeklyLqrAttachment;

class WeeklyLqrController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('weekly_lqr'), $filename);

            $weekly_lqr_attachment = new WeeklyLqrAttachment();
            $weekly_lqr_attachment->weekly_lqr_id = 0;
            $weekly_lqr_attachment->path = $filename;
            $weekly_lqr_attachment->token = $token;
            $weekly_lqr_attachment->save();

            $responseBody = $this->responseBody(true, "WeeklyLqrController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_weekly_lqrs.*,
            qc_weekly_lqr_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_weekly_lqr_attachments
            INNER JOIN
            qc_weekly_lqrs
            ON
                qc_weekly_lqr_attachments.weekly_lqr_id = qc_weekly_lqrs.weekly_lqr_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_weekly_lqrs.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                pp_customer_order_plans_data.product_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_weekly_lqrs.factory = hr_employee_sites.site_id
                GROUP BY
            qc_weekly_lqrs.weekly_lqr_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "weekly_lqr_id" => $record->weekly_lqr_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "WeeklyLqrController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $weeklyLqr = new WeeklyLqr();
            $weeklyLqr->user_id = Auth::user()->id;
            $weeklyLqr->reference_no = $request->get('reference_no');
            $weeklyLqr->date = $request->get('date');
            $weeklyLqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $weeklyLqr->factory = $request->get('factory');
            $weeklyLqr->description = $request->get('description');
            if ($weeklyLqr->save()) {
                $token = $request->get('attachment_token');
                $attachment = WeeklyLqrAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->weekly_lqr_id = $weeklyLqr->weekly_lqr_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "WeeklyLqrController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $weeklyLqr = WeeklyLqr::find($id);
            $weeklyLqr->user_id = Auth::user()->id;
            $weeklyLqr->reference_no = $request->get('reference_no');
            $weeklyLqr->date = $request->get('date');
            $weeklyLqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $weeklyLqr->factory = $request->get('factory');
            $weeklyLqr->description = $request->get('description');
            if ($weeklyLqr->update()) {
                $token = $request->get('attachment_token');
                $attachments = WeeklyLqrAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = WeeklyLqrAttachment::where('token', '=', $token)->first();
                        $attachment->weekly_lqr_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "WeeklyLqrController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = WeeklyLqrAttachment::where('weekly_lqr_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = WeeklyLqr::where('weekly_lqr_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = WeeklyLqrAttachment::where('weekly_lqr_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "WeeklyLqrController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_weekly_lqrs.*,
            qc_weekly_lqr_attachments.token
        FROM
            qc_weekly_lqr_attachments
            INNER JOIN
            qc_weekly_lqrs
            ON
                qc_weekly_lqr_attachments.weekly_lqr_id = qc_weekly_lqrs.weekly_lqr_id
        WHERE
            qc_weekly_lqrs.weekly_lqr_id = {$id}";

            $weeklyLqr =  DB::select($query);

            $responseBody = $this->responseBody(true, "WeeklyLqrController", "getData", $weeklyLqr[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "WeeklyLqrController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
